var fs		= require('fs');
var path	= require('path');
var sys		= require('sys');

exports.mounts = function( ){
	var returnObj = Array( );
	var currentMounts	= fs.readFileSync( "/etc/mtab", 'utf8' ).replace( RegExp( '\n$' ), '' ).split( "\n" );
	for( var x=0; x<currentMounts.length; x++ ){
		var currentMountParts	= currentMounts[x].split( " " );
		returnObj[x]		= new Array( );
		returnObj[x].device	= currentMountParts[0];		// This dot is ugly, and could potentially cause problems.
		returnObj[x].mountpoint	= currentMountParts[1].replace( RegExp( '.040' ), ' ' );
		returnObj[x].fs		= currentMountParts[2];
		returnObj[x].options	= currentMountParts[3];
	}
	
	return returnObj;
}

exports.ps = function( ){
	var returnArray	= Array( );
	try{
		var procContents	= fs.readdirSync( "/proc" );
		for( var x=0; x<procContents.length; x++ ){
			if( parseInt( procContents[x] ) ){ 
				// procContents[x] is a pid number..
				returnArray.push( getPidDetails( procContents[x] ) );
			}
		}
	}catch( err ){
		return false;
	}

	return returnArray;
}

exports.ps.pid	= function( pidToGet ){
	return getPidDetails( pidToGet );
}

function getPidDetails( pidNumber ){
	var returnArray = Array( );
	try{
		var statusLines	= fs.readFileSync( "/proc/" + pidNumber + "/status", 'utf8').split( "\n" );
		for( var x=0; x<statusLines.length; x++ ){
			var tmpSplit		= statusLines[x].split( ":" );
			// This if statement takes care of trailing \n's
			if( tmpSplit.length > 1 ){
				var tmpKey		= tmpSplit[0].toLowerCase( );
				// The status gives more number than we care about, seperated by tabs.. remove them.
				if( tmpKey == 'uid' || tmpKey == 'gid' ){
					returnArray[tmpKey]	= tmpSplit[1].trim().replace( RegExp( "\t.*" ), '' );
				}else{
					returnArray[tmpKey]	= tmpSplit[1].trim( );
				}
			}
		}
	}catch( err ){
		return false;
	}

	return returnArray;
}

exports.acpi	= function( ){

}

exports.acpi.batteries = function( ){
	var returnArray	= Array( );
	try{
		var batteries	= fs.readdirSync( "/proc/acpi/battery/", 'utf8' );
		for( var x=0; x<batteries.length; x++ ){
			var batteryName		= batteries[x];
			returnArray[x]		= new Array( );
		
			var batteryDetails	= getBatteryDetails( batteryName );
			batteryDetails['name']	= batteryName;

			returnArray[x]	= batteryDetails;
		}
	}catch( err ){
		return false;
	}

	return returnArray;
}

exports.acpi.battery = function( batteryName ){
	return getBatteryDetails( batteryName );
}

function getBatteryDetails( batteryName ){
	var returnArray	= Array( );
	try{ 
		var batteryStateLines	= fs.readFileSync( "/proc/acpi/battery/" + batteryName + "/state", 'utf8' ).split( "\n" );
		for( var x=0; x<batteryStateLines.length; x++ ){
			var tmpSplit	= batteryStateLines[x].split( ":" );
			if( tmpSplit.length > 1 ){
				var tmpKey		= tmpSplit[0].replace( RegExp( " " ), "_" ).trim( );
				returnArray[tmpKey]	= tmpSplit[1].trim( );
			}
		}
	}catch( err ){
		return false;
	}

	return returnArray;
}

exports.sysctl	= function( ){
	return recursiveSysctlObjFromDir( "/proc/sys/" );
}

function recursiveSysctlObjFromDir( currentDir ){
	
	var returnObj	= function( ){ };
	
	function makeSafeName( unsafeName ){
		if( typeof unsafeName == 'undefined' ){ return unsafeName; };
		return unsafeName	.replace( RegExp( '-', 'g' ), '_' )
					.replace( RegExp( '0', 'g' ), 'zero' )
					.replace( RegExp( '1', 'g' ), 'one' )
					.replace( RegExp( '2', 'g' ), 'two' )
					.replace( RegExp( '3', 'g' ), 'three' )
					.replace( RegExp( '4', 'g' ), 'four' )
					.replace( RegExp( '5', 'g' ), 'five' )
					.replace( RegExp( '6', 'g' ), 'six' )
					.replace( RegExp( '7', 'g' ), 'seven' )
					.replace( RegExp( '8', 'g' ), 'eight' )
					.replace( RegExp( '9', 'g' ), 'nine' )
					.replace( RegExp( '/', 'g' ), '_' );
	}

	function makeSafeContent( unsafeContent ){
		if( typeof unsafeContent == 'undefined' ){ return unsafeContent; };
		return unsafeContent.replace( RegExp( '\n', 'g' ), '' ).replace( '|', '' ).replace( "'", "\'" );
	}

	try{
		var tmpArray		= Array( );
		var contentsOfDir	= fs.readdirSync( currentDir, 'utf8' );
		// Loop through the contents of the given directory..
		for( var x=0; x<contentsOfDir.length; x++ ){
			var tmpStatObj		= fs.statSync( path.join( currentDir, contentsOfDir[x] ) );

			// If the given element in the directory is a file, recurse..
			if( tmpStatObj.isDirectory() ){
				returnObj[makeSafeName(contentsOfDir[x])]	= recursiveSysctlObjFromDir( path.join( currentDir, contentsOfDir[x] ) );
			}else{
			// The element in the directory is a file...

				var safeFileName	= makeSafeName( contentsOfDir[x] );
				var safeFileContent	= makeSafeContent( fs.readFileSync( path.join( currentDir, contentsOfDir[x] ), 'utf8' ) );
				var safeFullFileName		= makeSafeName( path.join( currentDir, contentsOfDir[x] ) );

				// This is the getter function..
				eval( "var get" + safeFullFileName + " = function( ){ return '" + safeFileContent + "'; }" );
				
				// This is the setter function..
				var setterString = "var set" + safeFullFileName + " = function( valueToSet ){";
				setterString 	+= "var writeStream = fs.createWriteStream( \"" + path.join( currentDir, contentsOfDir[x] ) + "\" );";
				setterString	+= "writeStream.on( 'error', function( err ){ return false; } );";
				setterString	+= "writeStream.end( valueToSet );";
				setterString	+= "return true;};";

				eval( setterString );

				// Getter function..
				returnObj.__defineGetter__( safeFileName, eval( "get" +safeFullFileName ) );

				// Setter function..
				returnObj.__defineSetter__( safeFileName, eval( "set" +safeFullFileName ) );
			}
		}

		return returnObj;
	}catch( err ){
		//console.log( "ERROR: " + err );
		return false;
	}
}
